<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Classifieds_model extends MY_Model{
 
    public $table_cfd 	= 'classifieds';
    public $table_cfdw	= 'classifieds_view'; 
    public $table_cat 	= 'categories';
    public $table_tags 	= 'tags';
    public $table_users	= 'users';    
    public $table_cfdc	= 'classified_answers';
	
	function __construct()
	{
		parent::__construct();
		$this->module = 'classifieds';
	}
	
	/**
	 * Gets all classifieds based on the $args parameter.
	 * 
	 * FORMAT - The $args parameter can be in the following formats:
	 * 
	 * 		1. As an array: 		array('key'=>'val', 'key2'=>'val2')
	 * 		2. As a query string: 	key=val&key2=val2
	 * 		
	 * 		Note: 	Some options that allow for multiple selections, ie category_in,
	 * 				are not supported via the query string method.
	 * 
	 * OPTIONS - The following options are available:
	 * 
	 * id (string|array)(optional)
	 *   Return classifieds with the specified id(s)
	 *   Default: none
	 * slug (string)(optional)
	 *   Return classified with the specified slug
	 *   Default: none
	 * published (bool)(optional)
	 * 	 Return all classifieds or only published classifieds
	 *   Default: TRUE
	 * category_in (string|int|array)(optional)
	 *   Return classifieds in specified category slug or category id. 
	 *   Default: none
	 * category_not_in (string|int|array)(optional)
	 *   Return classifieds NOT in specified category slug or category id.
	 *   Default: none
	 * tagged (string)(optional)
	 *   Return classifieds tagged with the specified tag
	 *   Default: none
	 * not_tagged (string)(optional)
	 *   Return classifieds NOT tagged with the specified tag
	 *   Default: none
	 * in_archive ( string(year) | array(year=>2009,month=>12) )(optional)
	 *   Return classifieds in a year/month based archive
	 *   Default: none
	 * order_by (string)(optional)
	 *   Return classifieds ordered by the specified order
	 *   Default: timestamp DESC
	 * limit (string)(optional)
	 * 	 Limit the number of return classifieds. also includes offset.
	 *   Default: 0, 100
	 * count (bool)(optional)
	 * 	 If this is true, the number of rows will be returned instead of a db result
	 *   Default: false
	 * 
	 * @param mixed $args 
	 * @return object
	 */
	function get( $args=NULL ){
		
		if(!is_array($args)){
			//extract to 0=>'key=val', 1=>'key=val' etc.
			$options = explode('&', $args);
			foreach($options as $index=>$str){
				//extract to key=>val
				$temp = explode('=', $str);
				$new_args[trim($temp[0])] = trim($temp[1]);
			}
			//copy new array over old string
			$args = $new_args;
		}

		//classified id
		if(isset($args['id']) && $args['id'] != NULL){
			$this->db->where_in('id', $args['id']);
		}
		//classified slug (1 classified only)
		if(isset($args['slug']) && $args['slug'] != NULL){
			$this->db->where_in('slug', $args['slug']);
		}
		//published
		if(!isset($args['published']) || $args['published'] == TRUE ){
			$this->db->where("timestamp < NOW() and status='Published'");
		}
		//categories in
		if(isset($args['category_in']) && $args['category_in'] != NULL){
			//check for category_id or category_slug
			$classifieds_field = (is_numeric($args['category_in']) || is_numeric($args['category_in'][0])) 
				? 'cat_id' : 'cat_slug';
			//id or slug in categories table
			$cats_field = substr($classifieds_field, 4);
				
			$where = "$classifieds_field IN ( SELECT $cats_field FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE (";
			if(is_array($args['category_in'])){
				//loop for multiple categories
				$where_count = 0;
				foreach($args['category_in'] as $cat){
					$where .= ($where_count++ > 0) ? ' OR ' : '';
					$where .= "lft BETWEEN (
							SELECT lft FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$cat'
						) AND (
							SELECT rgt FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$cat'
						)";
				}
			}
			else {
				//single category
				$where .= "lft BETWEEN (
							SELECT lft FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$args[category_in]'
						) AND (
							SELECT rgt FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$args[category_in]'
						)";
			}
			$where .= '))';
			$this->db->where($where, NULL, FALSE);
		}
		
		//categories not in
		if(isset($args['category_not_in']) && $args['category_not_in'] != NULL){
			//check for category_id or category_slug
			$classifieds_field = (is_numeric($args['category_not_in']) || is_numeric($args['category_not_in'][0])) 
				? 'cat_id' : 'cat_slug';
			//id or slug in categories table
			$cats_field = substr($classifieds_field, 4);
				
			$where = "$classifieds_field NOT IN ( SELECT $cats_field FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE (";
			if(is_array($args['category_in'])){
				//loop for multiple categories
				$where_count = 0;
				foreach($args['category_not_in'] as $cat){
					$where .= ($where_count++ > 0) ? ' OR ' : '';
					$where .= "lft BETWEEN (
							SELECT lft FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$cat'
						) AND (
							SELECT rgt FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$cat'
						)";
				}
			}
			else {
				//single category
				$where .= "lft BETWEEN (
							SELECT lft FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$args[category_not_in]'
						) AND (
							SELECT rgt FROM `" . $this->db->dbprefix($this->table_cat) . "` WHERE $cats_field='$args[category_not_in]'
						)";
			}
			$where .= '))';
			$this->db->where($where, NULL, FALSE);
		}
		
		//tagged
		if(isset($args['tagged']) && $args['tagged'] != NULL){
			$this->db->like('tags', $args['tagged']);
		}
		//not tagged
		if(isset($args['not_tagged']) && $args['not_tagged'] != NULL){
			$this->db->not_like('tags', $args['not_tagged']);
		}
		//in_archive
		if(isset($args['in_archive']) && $args['in_archive'] != NULL){
			if(is_array($args['in_archive'])){
				$this->db->where('YEAR(timestamp)', $args['in_archive']['year']);
				if($args['in_archive']['month'])
					$this->db->where('MONTH(timestamp)', $args['in_archive']['month']);
			} else {
				$this->db->where('YEAR(timestamp)', $args['in_archive']);
			}
		}
		//omit following criteria if only counting rows
		if(isset($args['count']) && $args['count'] == NULL || isset($args['count']) && $args['count'] === FALSE){
			//order by
			$this->db->order_by( (!$args['order_by']) ? 'timestamp desc' : $args['order_by'] );
			//limit/offset
			if(isset($args['limit']) && $args['limit'] != NULL)
				$this->db->limit($args['limit'],$args['offset']);
			else 
				$this->db->limit(100, 0);
		}
		
		if(isset($args['count']) && $args['count'] != NULL){
			$this->db->from($this->db->dbprefix($this->table_cfdw));
			return $this->db->count_all_results();
		} else {
			return $this->db->get($this->db->dbprefix($this->table_cfdw));
		}
	}
	
	
	/**
	 * Gets a list of archives by year and (optionally) month
	 *
	 * @param string $detail
	 * @param mixed $category
	 * @param string $type
	 * @return resource
	 */
	function getArchives($detail='month', $category=null, $type='IN'){
		if($detail == 'month')
			$grp = ", MONTH(timestamp)";
		
		if($category != null){
			if(is_array($category)){
				$i=0;
				foreach($category as $slug){
					$in .= ($i++ > 0) ? ',' : '';
					$in .= "'$slug'";
				}
				$opts = "AND cat_slug $type ($in)";
			} else {
				$opts = "AND cat_slug $type ('$category')";
			}
		}
			
		return $this->db->query("SELECT timestamp, count(*) cnt, MONTH(timestamp) month, YEAR(timestamp) year 
			FROM `" . $this->db->dbprefix($this->table_cfdw) . "` WHERE status='Published' $opts 
			GROUP BY YEAR(timestamp)$grp ORDER BY timestamp DESC");
	}
	
	
	function searchClassifieds($search_str){

		$where = " p.detail LIKE '%".$search_str."%'";			
		
		$q = "SELECT p.id, p.timestamp, DATE_FORMAT( p.timestamp, '%b %e, %Y' ) f_date, DATE_FORMAT( p.timestamp, '%l:%i%p' ) f_time, p.user_id, u.display_name, p.status, coalesce( p.title, 'Untitled' ) title, p.summary, p.detail, (
			SELECT count( * )
			FROM `" . $this->db->dbprefix($this->table_cfdc) . "`
			WHERE classified_id = p.id AND approved = 'true'
			)answer_count, p.allow_answers, p.answer_email_notify, p.tags, coalesce( c.category, 'Uncategorized' ) category, c.id cat_id, c.slug, p.service, p.service_id
			FROM `" . $this->db->dbprefix($this->table_cfdw) . "` p
			LEFT OUTER JOIN `" . $this->db->dbprefix($this->table_users) . "` u ON u.user_id = p.user_id
			LEFT OUTER JOIN `" . $this->db->dbprefix($this->table_cat) . "` c ON c.id = p.category
			WHERE ".$where." 
			ORDER BY timestamp DESC";
		return $this->db->query($q);
	}
	
	/**
	 * Save a classified.
	 *
	 * @param int $id
	 * @return mixed
	 */
	function save($id){
		//make slug
		$_POST['slug'] = url_title($_POST['title']);
		//post to twitter?
		$send_to_twitter = (isset($_POST['send_to_twitter'])) ? $_POST['send_to_twitter']:'';
		//with bitly url?
		$add_bitly_url = (isset($_POST['add_bitly_url'])) ? $_POST['add_bitly_url']:'';
		//remove from _POST array
		unset($_POST['send_to_twitter']);
		unset($_POST['add_bitly_url']);
		unset($_POST['new_cat_name']);
		unset($_POST['new_cat_parent']);
		
		if($id == NULL || $id == 0){
			//insert the new classified
			$save_result = $this->db->insert($this->db->dbprefix($this->table_cfd), $_POST);
			$id = $this->db->insert_id();
			
			//update any pending tags
			$this->db->update($this->db->dbprefix($this->table_tags), array('src_id'=>$id), array('src_id'=>0));
			
			//send to twitter
			if($send_to_twitter == 'on'){
				if($add_bitly_url == 'on'){
					//determine classified id type
					$classified_id = (config_item('bitly_classifieds_url_type') == 'id') ? $id : $_POST['slug'];
					//make the bit.ly url
					$url = $this->makeBitlyUrl(base_url().config_item('bitly_url_to_classifieds') . '/' . $classified_id);
					$status = substr($_POST['title'],0,116) . ' - ' . $url;
				} else {
					$status = substr($_POST['title'],0,140);
				}
				$this->tweet($status);
			}

		} else {
			$save_result = $this->db->update($this->db->dbprefix($this->table_cfd), $_POST, array('id'=>$id));
		}
		
		return $save_result;
	}
	
	/**
	 * Delete a classified
	 *
	 * @param int $id
	 * @return string
	 */
	function delete($id){
		//delete classified
		$res = $this->db->delete($this->db->dbprefix($this->table_cfd), array('id'=>$id));
		//delete answers
		$this->db->delete($this->db->dbprefix($this->table_cfdc), array('classified_id'=>$id));
		//delete tags
		$this->db->delete($this->db->dbprefix($this->table_tags), array('src_id'=>$id, 'tag_type'=>'classifieds'));
		
		return $res;
		/*
		if($res) 
			return "Classified deleted successfully.";
		else
			return "There was an error deleting your classified.";
		*/
	}
	
	/**
	 * Builds a 'tag cloud', returns an array of the details. 
	 * 
	 *
	 * @return array
	 */
	function getTagCloud(){
		$max = $this->db->query("SELECT tag, count(*) cnt FROM `" . $this->db->dbprefix($this->table_tags) . "`  
			WHERE tag_type = 'classifieds' GROUP BY tag ORDER BY cnt DESC")->first_row()->cnt;
		//determin size ranges
		$small = $max / 4;
		$medium = $small * 2;
		$large = $small * 3;
		$xlarge = $max;
		
		foreach($this->db->query("SELECT tag, slug, count(*) cnt FROM `" . $this->db->dbprefix($this->table_tags) . "`  
			WHERE tag_type='classifieds' GROUP BY tag ORDER BY tag")->result_array() as $row){
			if($row['cnt'] <= $small)
				$font_size = 10;
			elseif($row['cnt'] <= $medium)
				$font_size = 14;
			elseif($row['cnt'] <= $large)
				$font_size = 18;
			elseif($row['cnt'] <= $xlarge)
				$font_size = 22;
			else
				$font_size = 26;
		
			//put in array
			$arr[$row['tag']] = array('slug'=>$row['slug'],'size'=>$font_size,'count'=>$row['cnt']);
		}
		
		return $arr;
	}
	
	/*function getAllTags($id=NULL){
		return $this->db->query("SELECT tag, count(*) cnt from `" . $this->db->dbprefix($this->table_tags) . "` WHERE tag_type = 'classifieds' GROUP BY tag");
	}*/
   public function getAllTags( $limit,$offset){
      $this->db->select('tag, COUNT(*) AS cnt',FALSE);
      $this->db->from($this->db->dbprefix($this->table_tags));
      $this->db->where('tag_type','classifieds');
      $this->db->group_by('tag');
    $this->db->limit($limit,$offset);
	
      return $this->db->get();
	}

   //Count number of tags for classifieds
   public function countAllTags(){
      $nbre=$this->db->query("SELECT * from `" . $this->db->dbprefix($this->table_tags) . "` WHERE tag_type = 'classifieds'");
      
      return $nbre->num_rows();
     
     // return $this->db->count_all($this->db->dbprefix($this->table_tags));
   }

	
	function getTagsOnClassified($classified_id=0){
		return $this->db->get_where($this->db->dbprefix($this->table_tags), array('src_id' => $classified_id, 'tag_type'=>'classifieds'));
	}
	
	function addTags($classified_id){
		$tag_arr = explode(',', $_POST['tags']);
		
		foreach($tag_arr as $the_tag){
			$this->db->insert($this->db->dbprefix($this->table_tags), array(
				'src_id'=>$classified_id, 
				'tag_type'=>'classifieds', 
				'tag'=>trim($the_tag),
				'slug'=>strtolower(url_title($the_tag))
			));
		}
	}
	
	function deleteTag($tag_id){
		$this->db->delete($this->db->dbprefix($this->table_tags), array('id'=>$tag_id));
	}
	
	function getAnswers($id=null, $approved=false, $limit=50){
		if($id == null){
			return $this->db->query("SELECT c.id, c.classified_id, c.timestamp, c.name, c.url, c.email, 
				c.answer, c.ip, c.approved, p.title FROM `" . $this->db->dbprefix($this->table_cfdc) . "` c
				LEFT OUTER JOIN `" . $this->db->dbprefix($this->table_cfd) . "` p ON p.id = c.classified_id
				ORDER BY c.timestamp DESC LIMIT $limit");
		} else {
			if($approved) 
				return $this->db->get_where($this->db->dbprefix($this->table_cfdc), array('classified_id' => $id, 'approved' => 'true'));
			else 
				return $this->db->get_where($this->db->dbprefix($this->table_cfdc), array('classified_id' => $id));
		}
			
	}
	
	function getAnswer($id){
		return $this->db->get_where($this->db->dbprefix($this->table_cfdc), array('id' => $id));
	}
	
	function insertAnswer($data){
		
		$note = '';
		$row = $this->getClassifieds("id=$data[classified_id]")->first_row();
		//$approve_cmt = $this->options['approve_answer'];
		
		//make sure answers are allowed
		if($row->allow_answers == 'on'){
			//auto approve?
			if(config_item('approve_answer') == 'on'){
				$data['approved'] = 'false';
				$note = 'Your Answer is awaiting moderation.';
			}
			//too many url's?
			if(substr_count($data['answer'], 'http://') > 4)
				return "<img src='".base_url()."assets/themes/backoffice/images/error.png' /> Sorry, there are too many URL's in your Answer.";
			
			$this->db->insert($this->db->dbprefix($this->table_cfdc), $data);
			
			return '<img src="'.base_url().'assets/themes/backoffice/images/tick.png" /> Thanks for your Answer! ' . (isset($note)) ? $note : '';
		}
		else 
			return "<img src='".base_url()."assets/themes/backoffice/images/error.png' /> Sorry, Answers are disabled for this classified.";
	}
	
	function saveAnswer($id){
		return $this->db->update($this->db->dbprefix($this->table_cfdc), $_POST, array('id'=>$id));
	}
	
	function deleteAnswer($id){
		return $this->db->delete($this->db->dbprefix($this->table_cfdc), array('id'=>$id));
	}
	
	function approveAnswer($id, $approved){
		$this->db->update($this->db->dbprefix($this->table_cfdc), array('approved'=>$approved), array('id'=>$id));
	}
	
	function unapprovedAnswerCount(){
		return $this->db->query("SELECT count(*) cnt FROM `" . $this->db->dbprefix($this->table_cfdc) . "` 
			WHERE approved != 'true'")->first_row()->cnt;
	}
	
	function getTweets(){
		return $this->db->query("SELECT * FROM `" . $this->db->dbprefix($this->table_cfd) . "` WHERE service='twitter' ORDER BY timestamp DESC");
	}
	
	function tweet($status=NULL){
		if(config_item('twitter_integration') != 'on')
			return 'Twitter integration is not enabled.';
		
		$this->load->library('twitter', 
			array('user'=>config_item('twitter_username'), 'pass'=>config_item('twitter_password'))
		);
		
		$status = ($status == NULL) ? $_POST['twitter_stat'] : $status;
		
		$result = $this->twitter->setStatus($status);
		if($result !== FALSE){
			if($_POST['add_as_classifieds'] == 'on')
				$this->db->insert($this->db->dbprefix($this->table_cfd), array('user'=>USER, 'title'=>$_POST['twitter_stat'], 'service'=>'twitter', 'service_id'=>$result));
			
			return 'Update posted to twitter.';
		}
		else 
			return 'Error: Unable to update twitter.';
	}
	
	function getRecentTweets(){
		if(config_item('twitter_integration') != 'on')
			return 'Twitter integration is not enabled.';
		
		$this->load->library('twitter', 
			array('user'=>config_item('twitter_username'), 'pass'=>config_item('twitter_password'))
		);
		$result = $this->twitter->getStatus();
		if($result !== FALSE){
			return $result;
		}
		else 
			return 'Error: Unable to connect to twitter.';
	}
	
	function makeBitlyUrl($url, $login=null, $apiKey=null, $format='json', $version='2.0.1'){
		if($this->options['bitly_integration'] != 'on')
			return 'Bit.ly integration is not enabled.';
		
		if($login == null)
			$login = $this->getOption('bitly_login');
		if($apiKey == null)
			$apiKey = $this->getOption('bitly_api_key');
		
		$bitly = 'http://api.bit.ly/shorten?version=2.0.1&login=' . $login .
			'&apiKey=' . $apiKey . '&longUrl=' . $url . '&format=' . $format;
		
		$json = json_decode(file_get_contents($bitly), true);
		
		return $json['results'][$url]['shortUrl'];
	}
	
	/**
	 * Get a single classified by id or slug
	 * 
	 * @deprecated use get('id=3')
	 * 
	 * @param int $id
	 * @return array
	 */
	function getClassified($id){
		if(is_numeric($id))
			return $this->db->get_where($this->db->dbprefix($this->table_cfdw), array('id' => $id));
		else
			return $this->db->get_where($this->db->dbprefix($this->table_cfdw), array('slug'=>$id))->first_row('array');
	}
	
	/**
	 * Get classifieds
	 * 
	 * @deprecated use get($args)
	 *
	 * @param unknown_type $args
	 * @return unknown
	 */
	function getClassifieds( $args=NULL ){
		return $this->get($args);
	}
	
	/**
	 * Get a count of classifieds
	 *
	 * @deprecated use getClassifieds('count=1')
	 * 
	 * @param bool $published
	 * @return int
	 */
	function countClassifieds($published=TRUE){
		if($published)
			return $this->db->count_all($this->db->dbprefix($this->table_cfdw));
		else
			return $this->db->query("
				SELECT count(*) cnt 
				FROM `" . $this->db->dbprefix($this->table_cfdw) . "`  
				WHERE timestamp < NOW() 
				AND status='Published'
			")->first_row()->cnt;
	}
}
